/**
 * Copyright 2005 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 *
 * @author Fuad Efendi
 */
package org.apache.nutch.protocol.httpclient.innovation;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.nutch.db.Page;
import org.apache.nutch.pagedb.FetchListEntry;
import org.apache.nutch.protocol.Content;
import org.apache.nutch.protocol.Protocol;
import org.apache.nutch.protocol.ProtocolOutput;
import org.apache.nutch.protocol.ProtocolStatus;
import org.apache.nutch.protocol.ResourceGone;
import org.apache.nutch.util.LogFormatter;
import org.apache.nutch.util.NutchConf;
import org.apache.nutch.util.mime.MimeType;
import org.apache.nutch.util.mime.MimeTypes;

import HTTPClient.HTTPConnection;
import HTTPClient.HTTPResponse;
import HTTPClient.ModuleException;
import HTTPClient.NVPair;

/**
 * An implementation of HTTP protocol
 * @author Fuad Efendi
 */
public class Http implements Protocol {

	public static final Logger LOG = LogFormatter
			.getLogger("org.apache.nutch.net.Http");

	static {
		if (NutchConf.get().getBoolean("http.verbose", false))
			LOG.setLevel(Level.FINE);
	}

	public ProtocolOutput getProtocolOutput(String url) {
		try {
			return getProtocolOutput(new FetchListEntry(true, new Page(url, 1.0f),
					new String[0]));
		} catch (MalformedURLException mue) {
			return new ProtocolOutput(null, new ProtocolStatus(mue));
		}
	}

	static String AGENT_STRING = getAgentString();

	static NVPair[] AGENT_HEADER = new NVPair[1];
	static {
		if ((AGENT_STRING == null) || (AGENT_STRING.length() == 0)) {
			LOG.severe("User-agent is not set!");
		} else {
			AGENT_HEADER[0] = new NVPair("User-Agent", AGENT_STRING);
		}
	}

	public ProtocolOutput getProtocolOutput(FetchListEntry fle) {

		String urlString = fle.getUrl().toString();
		try {

			URL url = new URL(urlString);

      try {
        if (!RobotRulesParser.isAllowed(url))
                return new ProtocolOutput(null, new ProtocolStatus(ProtocolStatus.ROBOTS_DENIED, url));
      } catch (Throwable e) {
        // XXX Maybe bogus: assume this is allowed.
        LOG.info("Exception checking robot rules for " + url + ": " + e);
      }


			HTTPConnection con = HttpFactory.getConnection(url.getHost());
			HTTPResponse response = con.Get(url.getPath(), AGENT_HEADER);
			byte[] data;

			int code = response.getStatusCode();

			if (code == 200) { // got a good response
				return new ProtocolOutput(new Content(urlString, urlString, response
						.getData(), getContentType(urlString, response),
						getHeaders(response))); // return it

				// public Content(String url, String base, byte[] content, String
				// contentType,
				// Properties metadata){

			} else if (code == 410) { // page is gone
				throw new ResourceGone(url, "Http: " + code);

				// Redirects
			} else if (code >= 300 && code < 400) {

				/**
				 * Redirections (status codes 301, 302, 303, 305 and 307) are handled
				 * automatically for the request methods GET and HEAD;
				 * The final URI that delivered the
				 * response can be retrieved using the response's getEffectiveURI()
				 * method.
				 */

				LOG.fine("redirect to " + response.getEffectiveURI());

			} else { // convert to exception
				throw new HttpError(code);
			}

		} catch (Exception e) {
			return new ProtocolOutput(null, new ProtocolStatus(e));
		}

		return null;

	}

	/** A flag that tells if magic resolution must be performed */
	private final static boolean MAGIC = NutchConf.get().getBoolean(
			"mime.type.magic", true);

	/** Get the MimeTypes resolver instance. */
	private final static MimeTypes MIME = MimeTypes.get(NutchConf.get().get(
			"mime.types.file"));

	private String getContentType(String urlString, HTTPResponse response)
			throws IOException, ModuleException {

		String contentType = response.getHeader("Content-Type");
		if (contentType == null) {
			MimeType type = null;
			if (MAGIC) {
				type = MIME.getMimeType(urlString, response.getData());
			} else {
				type = MIME.getMimeType(urlString);
			}
			if (type != null) {
				contentType = type.getName();
			} else {
				contentType = "";
			}
		}

		return contentType;
	}

	private Properties getHeaders(HTTPResponse response) throws IOException,
			ModuleException {

		Properties headers = new Properties();

		Enumeration hdr_list = response.listHeaders();
		while (hdr_list.hasMoreElements()) {
			String header = (String) hdr_list.nextElement();

			String value = response.getHeader(header);

			headers.put(header, value);

		}

		return headers;
	}

	private static String getAgentString() {

		String agentName = NutchConf.get().get("http.agent.name");
		String agentVersion = NutchConf.get().get("http.agent.version");
		String agentDesc = NutchConf.get().get("http.agent.description");
		String agentURL = NutchConf.get().get("http.agent.url");
		String agentEmail = NutchConf.get().get("http.agent.email");

		if ((agentName == null) || (agentName.trim().length() == 0))
			LOG.severe("No User-Agent string set (http.agent.name)!");

		StringBuffer buf = new StringBuffer();

		buf.append(agentName);
		if (agentVersion != null) {
			buf.append("/");
			buf.append(agentVersion);
		}
		if (((agentDesc != null) && (agentDesc.length() != 0))
				|| ((agentEmail != null) && (agentEmail.length() != 0))
				|| ((agentURL != null) && (agentURL.length() != 0))) {
			buf.append(" (");

			if ((agentDesc != null) && (agentDesc.length() != 0)) {
				buf.append(agentDesc);
				if ((agentURL != null) || (agentEmail != null))
					buf.append("; ");
			}

			if ((agentURL != null) && (agentURL.length() != 0)) {
				buf.append(agentURL);
				if (agentEmail != null)
					buf.append("; ");
			}

			if ((agentEmail != null) && (agentEmail.length() != 0))
				buf.append(agentEmail);

			buf.append(")");
		}
		return buf.toString();
	}

}
